@@ -15,6 +15,7 @@
 #include <xf86drmMode.h>
 
 #include "drm_info.h"
+#include "tables.h"
 
 static const struct {
 	const char *name;
@@ -47,6 +48,39 @@ static const struct {
 	{ "SYNCOBJ_TIMELINE", DRM_CAP_SYNCOBJ_TIMELINE },
 };
 
+static struct json_object *tainted_info(void)
+{
+#ifndef __linux__
+	return NULL;
+#endif
+
+	FILE *f = fopen("/proc/sys/kernel/tainted", "r");
+	if (f == NULL) {
+		perror("fopen(/proc/sys/kernel/tainted)");
+		return NULL;
+	}
+
+	char str[64];
+	size_t n = fread(str, 1, sizeof(str) - 1, f);
+	if (!feof(f)) {
+		fclose(f);
+		fprintf(stderr, "fread(/proc/sys/kernel/tainted) failed");
+		return NULL;
+	}
+	str[n] = '\0';
+
+	fclose(f);
+
+	errno = 0;
+	unsigned long tainted = strtoull(str, NULL, 10);
+	if (errno != 0) {
+		perror("strtoull");
+		return NULL;
+	}
+
+	return json_object_new_uint64(tainted);
+}
+
 static struct json_object *kernel_info(void)
 {
 	struct utsname utsname;
@@ -62,6 +96,7 @@ static struct json_object *kernel_info(void)
 		json_object_new_string(utsname.release));
 	json_object_object_add(obj, "version",
 		json_object_new_string(utsname.version));
+	json_object_object_add(obj, "tainted", tainted_info());
 	return obj;
 }
 
@@ -129,39 +164,82 @@ static struct json_object *device_info(int fd)
 	json_object_object_add(obj, "bus_type",
 		json_object_new_uint64(dev->bustype));
 
-	struct json_object *device_data_obj = NULL;
+	struct json_object *device_data_obj = NULL, *bus_data_obj = NULL;
+	struct json_object *compatible_arr = NULL;
 	switch (dev->bustype) {
 	case DRM_BUS_PCI:;
-		drmPciDeviceInfo *pci = dev->deviceinfo.pci;
+		drmPciDeviceInfo *pci_dev = dev->deviceinfo.pci;
+		drmPciBusInfo *pci_bus = dev->businfo.pci;
+
 		device_data_obj = json_object_new_object();
 		json_object_object_add(device_data_obj, "vendor",
-			json_object_new_uint64(pci->vendor_id));
+			json_object_new_uint64(pci_dev->vendor_id));
 		json_object_object_add(device_data_obj, "device",
-			json_object_new_uint64(pci->device_id));
+			json_object_new_uint64(pci_dev->device_id));
 		json_object_object_add(device_data_obj, "subsystem_vendor",
-			json_object_new_uint64(pci->subvendor_id));
+			json_object_new_uint64(pci_dev->subvendor_id));
 		json_object_object_add(device_data_obj, "subsystem_device",
-			json_object_new_uint64(pci->subdevice_id));
+			json_object_new_uint64(pci_dev->subdevice_id));
+
+		bus_data_obj = json_object_new_object();
+		json_object_object_add(bus_data_obj, "domain",
+			json_object_new_uint64(pci_bus->domain));
+		json_object_object_add(bus_data_obj, "bus",
+			json_object_new_uint64(pci_bus->bus));
+		json_object_object_add(bus_data_obj, "slot",
+			json_object_new_uint64(pci_bus->dev));
+		json_object_object_add(bus_data_obj, "function",
+			json_object_new_uint64(pci_bus->func));
 		break;
 	case DRM_BUS_USB:;
-		drmUsbDeviceInfo *usb = dev->deviceinfo.usb;
+		drmUsbDeviceInfo *usb_dev = dev->deviceinfo.usb;
+		drmUsbBusInfo *usb_bus = dev->businfo.usb;
+
 		device_data_obj = json_object_new_object();
 		json_object_object_add(device_data_obj, "vendor",
-			json_object_new_uint64(usb->vendor));
+			json_object_new_uint64(usb_dev->vendor));
 		json_object_object_add(device_data_obj, "product",
-			json_object_new_uint64(usb->product));
+			json_object_new_uint64(usb_dev->product));
+
+		bus_data_obj = json_object_new_object();
+		json_object_object_add(bus_data_obj, "bus",
+			json_object_new_uint64(usb_bus->bus));
+		json_object_object_add(bus_data_obj, "device",
+			json_object_new_uint64(usb_bus->dev));
 		break;
 	case DRM_BUS_PLATFORM:;
-		drmPlatformDeviceInfo *platform = dev->deviceinfo.platform;
+		drmPlatformDeviceInfo *platform_dev = dev->deviceinfo.platform;
+		drmPlatformBusInfo *platform_bus = dev->businfo.platform;
+
 		device_data_obj = json_object_new_object();
-		struct json_object *compatible_arr = json_object_new_array();
-		for (size_t i = 0; platform->compatible[i]; ++i)
+		compatible_arr = json_object_new_array();
+		for (size_t i = 0; platform_dev->compatible[i]; ++i)
 			json_object_array_add(compatible_arr,
-				json_object_new_string(platform->compatible[i]));
+				json_object_new_string(platform_dev->compatible[i]));
 		json_object_object_add(device_data_obj, "compatible", compatible_arr);
+
+		bus_data_obj = json_object_new_object();
+		json_object_object_add(bus_data_obj, "fullname",
+			json_object_new_string(platform_bus->fullname));
+		break;
+	case DRM_BUS_HOST1X:;
+		drmHost1xDeviceInfo *host1x_dev = dev->deviceinfo.host1x;
+		drmHost1xBusInfo *host1x_bus = dev->businfo.host1x;
+
+		device_data_obj = json_object_new_object();
+		compatible_arr = json_object_new_array();
+		for (size_t i = 0; host1x_dev->compatible[i]; ++i)
+			json_object_array_add(compatible_arr,
+				json_object_new_string(host1x_dev->compatible[i]));
+		json_object_object_add(device_data_obj, "compatible", compatible_arr);
+
+		bus_data_obj = json_object_new_object();
+		json_object_object_add(bus_data_obj, "fullname",
+			json_object_new_string(host1x_bus->fullname));
 		break;
 	}
 	json_object_object_add(obj, "device_data", device_data_obj);
+	json_object_object_add(obj, "bus_data", bus_data_obj);
 
 	drmFreeDevice(&dev);
 
@@ -288,6 +366,69 @@ static struct json_object *path_info(int fd, uint32_t blob_id)
 	return obj;
 }
 
+static struct json_object *hdr_output_metadata_info(int fd, uint32_t blob_id)
+{
+	drmModePropertyBlobRes *blob = drmModeGetPropertyBlob(fd, blob_id);
+	if (!blob) {
+		perror("drmModeGetPropertyBlob");
+		return NULL;
+	}
+
+	struct json_object *obj = NULL;
+
+	// The type field in the struct comes first and is an u32
+	if (blob->length < sizeof(uint32_t)) {
+		fprintf(stderr, "HDR output metadata blob too short\n");
+		goto exit;
+	}
+
+	const struct hdr_output_metadata *meta = blob->data;
+
+	obj = json_object_new_object();
+	json_object_object_add(obj, "type", json_object_new_uint64(meta->metadata_type));
+
+	if (meta->metadata_type == HDMI_STATIC_METADATA_TYPE1) {
+		const size_t min_size = offsetof(struct hdr_output_metadata, hdmi_metadata_type1)
+			+ sizeof(struct hdr_metadata_infoframe);
+		if (blob->length < min_size) {
+			fprintf(stderr, "HDR output metadata blob too short\n");
+			goto exit;
+		}
+
+		const struct hdr_metadata_infoframe *info = &meta->hdmi_metadata_type1;
+		json_object_object_add(obj, "eotf", json_object_new_int(info->eotf));
+		// TODO: maybe add info->metadata_type, but seems to be the same as
+		// meta->metadata_type?
+		struct json_object *dp_obj = json_object_new_object();
+		static const char *dp_keys[] = {"r", "g", "b"};
+		for (size_t i = 0; i < 3; i++) {
+			struct json_object *coord_obj = json_object_new_object();
+			json_object_object_add(coord_obj, "x",
+				json_object_new_double(info->display_primaries[i].x / 50000.0));
+			json_object_object_add(coord_obj, "y",
+				json_object_new_double(info->display_primaries[i].y / 50000.0));
+			json_object_object_add(dp_obj, dp_keys[i], coord_obj);
+		}
+		json_object_object_add(obj, "display_primaries", dp_obj);
+		struct json_object *coord_obj = json_object_new_object();
+		json_object_object_add(coord_obj, "x",
+			json_object_new_double(info->white_point.x / 50000.0));
+		json_object_object_add(coord_obj, "y",
+			json_object_new_double(info->white_point.y / 50000.0));
+		json_object_object_add(obj, "white_point", coord_obj);
+		json_object_object_add(obj, "max_display_mastering_luminance",
+			json_object_new_int(info->max_display_mastering_luminance));
+		json_object_object_add(obj, "min_display_mastering_luminance",
+			json_object_new_double(info->min_display_mastering_luminance / 10000.0));
+		json_object_object_add(obj, "max_cll", json_object_new_int(info->max_cll));
+		json_object_object_add(obj, "max_fall", json_object_new_int(info->max_fall));
+	}
+
+exit:
+	drmModeFreePropertyBlob(blob);
+	return obj;
+}
+
 static struct json_object *fb_info(int fd, uint32_t id)
 {
 #ifdef HAVE_GETFB2
@@ -456,6 +597,8 @@ static struct json_object *properties_info(int fd, uint32_t id, uint32_t type)
 				data_obj = writeback_pixel_formats_info(fd, value);
 			} else if (strcmp(prop->name, "PATH") == 0) {
 				data_obj = path_info(fd, value);
+			} else if (strcmp(prop->name, "HDR_OUTPUT_METADATA") == 0) {
+				data_obj = hdr_output_metadata_info(fd, value);
 			}
 			break;
 		case DRM_MODE_PROP_RANGE:
