@@ -40,6 +40,16 @@ static uint64_t get_object_object_uint64(struct json_object *obj,
 	return json_object_get_uint64(uint64_obj);
 }
 
+static double get_object_object_double(struct json_object *obj,
+		const char *key)
+{
+	struct json_object *double_obj = json_object_object_get(obj, key);
+	if (!double_obj) {
+		return 0;
+	}
+	return json_object_get_double(double_obj);
+}
+
 static void print_driver(struct json_object *obj)
 {
 	const char *name = get_object_object_string(obj, "name");
@@ -118,6 +128,7 @@ static void print_device(struct json_object *obj)
 	int available_nodes = get_object_object_uint64(obj, "available_nodes");
 	int bus_type = get_object_object_uint64(obj, "bus_type");
 	struct json_object *data_obj = json_object_object_get(obj, "device_data");
+	struct json_object *compatible_arr = NULL;
 
 	printf(L_VAL "Device: %s", bustype_str(bus_type));
 	switch (bus_type) {
@@ -143,8 +154,14 @@ static void print_device(struct json_object *obj)
 		printf(" %04x:%04x", usb_vendor, usb_product);
 		break;
 	case DRM_BUS_PLATFORM:;
-		struct json_object *compatible_arr =
-			json_object_object_get(data_obj, "compatible");
+		compatible_arr = json_object_object_get(data_obj, "compatible");
+		for (size_t i = 0; i < json_object_array_length(compatible_arr); i++) {
+			printf(" %s", json_object_get_string(
+				json_object_array_get_idx(compatible_arr, i)));
+		}
+		break;
+	case DRM_BUS_HOST1X:;
+		compatible_arr = json_object_object_get(data_obj, "compatible");
 		for (size_t i = 0; i < json_object_array_length(compatible_arr); i++) {
 			printf(" %s", json_object_get_string(
 				json_object_array_get_idx(compatible_arr, i)));
@@ -158,7 +175,7 @@ static void print_device(struct json_object *obj)
 	printf("\n");
 }
 
-// The refresh rate provided by the mode itself is innacurate,
+// The refresh rate provided by the mode itself is inaccurate,
 // so we calculate it ourself.
 static int32_t refresh_rate(struct json_object *obj) {
 	int clock = get_object_object_uint64(obj, "clock");
@@ -367,6 +384,66 @@ static void print_path(struct json_object *obj, const char *prefix)
 	printf("%s" L_LAST "%s\n", prefix, json_object_get_string(obj));
 }
 
+static void print_hdr_output_metadata(struct json_object *obj,
+		const char *prefix)
+{
+	int type = get_object_object_uint64(obj, "type");
+
+	if (type != HDMI_STATIC_METADATA_TYPE1) {
+		printf("%s" L_LAST "Type: Reserved (%d)\n", prefix, type);
+		return;
+	}
+	printf("%s" L_VAL "Type: Static Metadata Type 1\n", prefix);
+
+	int eotf = get_object_object_uint64(obj, "eotf");
+	printf("%s" L_VAL "EOTF: ", prefix);
+	switch (eotf) {
+	case CTA_EOTF_TRADITIONAL_SDR:
+		printf("Traditional gamma - SDR");
+		break;
+	case CTA_EOTF_TRADITIONAL_HDR:
+		printf("Traditional gamma - HDR");
+		break;
+	case CTA_EOTF_SMPTE_2084:
+		printf("SMPTE ST 2084 (PQ)");
+		break;
+	case CTA_EOTF_HLG:
+		printf("HLG");
+		break;
+	default:
+		printf("Reserved (%d)", eotf);
+		break;
+	}
+	printf("\n");
+
+	struct json_object *dp_obj = json_object_object_get(obj, "display_primaries");
+	static const char *dp_keys[] = {"r", "g", "b"};
+	static const char *dp_names[] = {"Red", "Green", "Blue"};
+	printf("%s" L_VAL "Display primaries:\n", prefix);
+	for (size_t i = 0; i < 3; i++) {
+		struct json_object *coord_obj = json_object_object_get(dp_obj, dp_keys[i]);
+		printf("%s" L_LINE "%s%s: ", prefix, i == 2 ? L_LAST : L_VAL, dp_names[i]);
+		printf("(%.4f, %.4f)\n",
+			get_object_object_double(coord_obj, "x"),
+			get_object_object_double(coord_obj, "y"));
+	}
+
+	struct json_object *wp_obj = json_object_object_get(obj, "white_point");
+	printf("%s" L_VAL "White point: ", prefix);
+	printf("(%.4f, %.4f)\n",
+		get_object_object_double(wp_obj, "x"),
+		get_object_object_double(wp_obj, "y"));
+
+	printf("%s" L_VAL "Max display mastering luminance: %d cd/m²\n", prefix,
+		(int) get_object_object_uint64(obj, "max_display_mastering_luminance"));
+	printf("%s" L_VAL "Min display mastering luminance: %.4f cd/m²\n", prefix,
+		get_object_object_double(obj, "min_display_mastering_luminance"));
+	printf("%s" L_VAL "Max content light level: %d cd/m²\n", prefix,
+		(int) get_object_object_uint64(obj, "max_cll"));
+	printf("%s" L_LAST "Max frame average light level: %d cd/m²\n", prefix,
+		(int) get_object_object_uint64(obj, "max_fall"));
+}
+
 static void print_fb(struct json_object *obj, const char *prefix)
 {
 	uint32_t id = get_object_object_uint64(obj, "id");
@@ -517,6 +594,8 @@ static void print_properties(struct json_object *obj, const char *prefix)
 				print_writeback_pixel_formats(data_obj, sub_prefix);
 			else if (strcmp(prop_name, "PATH") == 0)
 				print_path(data_obj, sub_prefix);
+			else if (strcmp(prop_name, "HDR_OUTPUT_METADATA") == 0)
+				print_hdr_output_metadata(data_obj, sub_prefix);
 			break;
 		case DRM_MODE_PROP_BITMASK:
 			printf("bitmask {");
